/*******************
 ** IgnoreWS.java **
 *******************
 ** @author Ben Y. Zhao (ravenben@cs.berkeley.edu)
 ** @version: Release 1.1 CVS:$Revision: 1.4 $ $Date: 1999/09/10 00:12:33 $
 ** (c) 1999 by Ben Yanbin Zhao and the Regents of the University of California
 ** <P>
 ** This is a utility class used by XSet to strip out "ignorable whitespace" in
 ** the DOM tree.  Ignoreable whitespace by my definition, is tabs, spaces, and
 ** form feed and carriage return characters between child nodes.  They are not
 ** ignorable if the text node also contains non-space characters.
 **/

package ninja.xset;

import org.w3c.dom.*;
import com.ibm.xml.parsers.DOMParser;
import java.util.*;

public class IgnoreWS {
  
  public IgnoreWS() {
  }
  
  /**
   ** Takes a dom element in, and returns an array of non whitespace
   ** child elements
   **/
  public static Vector getChildren(Element e) {
    Node a = null, b = null;
    if ((a = e.getFirstChild()) == null) {
      return null;
    } else {
      Vector children = new Vector();
      //Standard parser eliminates comments in parsing, so we don't
      //worry about it here
      if (a.getNodeType() != Node.TEXT_NODE) {
	children.addElement(a);
      }
      while ((a = a.getNextSibling()) != null) {
	if (a.getNodeType() != Node.TEXT_NODE) {
	  children.addElement(a);
	} else {
	  if (!ignorable(a)) children.addElement(a);
	}
      }
      if (children.isEmpty()) return null;
      return children;
    }
  }
  
  private static boolean ignorable(Node e) {
    String temp = e.getNodeValue();
    return (temp.trim().length() == 0);
  }
  
  public static void main (String [] argv) {
    try {
      DOMParser parser = new DOMParser();
      parser.parse("test.xml");
      Document doc = parser.getDocument();
      Element e = doc.getDocumentElement();
      printElement(e);
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
  
  public static void printElement(Element n) {
    System.out.println("Begin tag: "+n.getNodeName()+"\n");
    System.out.println("Value: "+n.getNodeValue()+"\n");
    Vector children = getChildren(n);
    if (children != null) {
      Enumeration e = children.elements();
      int i = 0;
      Element ec = null;
      while (e.hasMoreElements()) {
	System.out.println("child #"+(++i));
	ec = (Element) e.nextElement();
	System.out.println("  has children?"+ec.hasChildNodes());
	System.out.println("  Child Tag: "+ec.getNodeName());
	System.out.println("  Value: >>"+ec.getNodeValue()+"<<");
      }
    }
  }
}
