/**
 ** @author: Ben Y. Zhao (ravenben@cs.berkeley.edu)
 ** @version: Release 1.1 CVS:$Revision: 1.8 $ $Date: 1999/09/10 00:12:34 $
 ** (c) 1999 by Ben Yanbin Zhao and the Regents of the University of California
 ** <P>
 ** This is the wrapper class that surrounds the actual XML file.
 **/

package ninja.xset;
import org.w3c.dom.*;
import java.util.*;

public class ServiceEntry {
  private String id = null;
  private String description = null;
  //  private Hashtable tagValues = null;
  private long lastHeartBeat = 0;
  private Vector hashtables = null;


  /**
   ** NOTE:  There could be problems in the hashing of identical XML
   ** but with different spacing into different service IDs
   ** Note that the hashing of the DOM object is actually hashing a
   ** canonical XML output that I generate.  Should fix this by
   ** adding a canonical XML transformer in SETutils.
   **/
  public ServiceEntry(Document d, long lastHeartBeat) 
    throws XSetException {
    try {
      this.id = SETutils.hasher(d, "SHA-1");
    } catch (java.security.NoSuchAlgorithmException nsae) {
      nsae.printStackTrace(System.err);
      throw new XSetException("Missing SHA-1 hash algorithm!");
    }
    this.description = XMLHelper.DocumentToString(d);
    this.lastHeartBeat = lastHeartBeat;
    this.hashtables = new Vector();
  }

  public ServiceEntry(String xmltext, long lastHeartBeat) 
    throws XSetException {
    try {
      this.id = SETutils.hasher(xmltext, "SHA-1");
    } catch (java.security.NoSuchAlgorithmException nsae) {
      nsae.printStackTrace(System.err);
      throw new XSetException("Missing SHA-1 hash algorithm!");
    }
    this.description = xmltext;
    this.lastHeartBeat = lastHeartBeat;
    this.hashtables = new Vector();
  }
  
  public void updateTime(long newHeartBeat) {
    lastHeartBeat = newHeartBeat;
  }
  public long getHeartBeat() {
    return lastHeartBeat;
  }
  public String getid() {
    return id;
  }
  public void addHashtable(Hashtable h) {
    hashtables.addElement(h);
  }
  public Vector getHashList() {
    return hashtables;
  }
  public String toString() {
    String out = "";
    out = out+"XML:\n"+description+"\nLast Beat:\n"+lastHeartBeat+"\n";
    return out;
  }
  public String getXML() {
    return description;
  }
}
