// SwingDemoApp.java

/****************************************************************************
 *** SwingDemoApp
 ** The Swing Application used as a graphical Demo for XSet's capabilities
 ** @author Ben Y. Zhao
 ** @version: Release 1.1 CVS:$Revision: 1.7 $ $Date: 1999/09/10 00:12:34 $
 ** (c) 1999 by Ben Yanbin Zhao and the Regents of the University of California
 ***************************************************************************/

package ninja.xset;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import ninja.ispace.*;

public class SwingDemoApp extends JFrame
  implements ActionListener, ListSelectionListener {
  
  JTextArea output;
  JScrollPane scrollPane;
  JSplitPane splitPane;
  DefaultListModel DTDNames = new DefaultListModel();
  DefaultListModel queryTagModel = new DefaultListModel();
  String DTDPath = null;
  Hashtable DTDContents = new Hashtable();
  Container contentPane;
  JList list;
  JScrollPane listScrollPane;
  JTextArea DTDTextArea; 
  JScrollPane DTDScrollPane;
  JTextArea query;
  JScrollPane queryPane;
  
  iSpaceLoaderIF loader;
  XSetServiceIF xset;
  
  public SwingDemoApp(String host) {
    //First: get Ninja loader
    try {
      loader = (iSpaceLoaderIF)iSpaceLoader.getLoaderStub(host);
    } catch (Exception e) {
      System.err.println("Can't get NINJA Loader: "+e);
      System.exit(-1);
    }
    //Second: grab XSet Service
    try {
      xset = (XSetServiceIF) loader.getService("ninja.xset.XSetService");
      if (xset == null) {
	System.err.println("Error getting XSetService.");
	System.exit(-1);
      }
    } catch (Exception e) {
      System.err.println("Runtime exception: "+e);
      e.printStackTrace();
      System.exit(-1);
    }
    
    // Set look and feel
    try {
      UIManager.setLookAndFeel(
	      UIManager.getCrossPlatformLookAndFeelClassName());
    } catch (Exception e) {}
    
    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
	System.exit(0);
      }
    });
    
    contentPane = getContentPane();
    
    //adding DTD viewer SplitPane
    //setting DTD list
    
    list = new JList(DTDNames);
    list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    list.addListSelectionListener(this);
    listScrollPane = new JScrollPane(list);
    TitledBorder tb = BorderFactory.createTitledBorder(
	              BorderFactory.createEtchedBorder(), "DTD List");
    tb.setTitleJustification(TitledBorder.CENTER);
    listScrollPane.setBorder(BorderFactory.createCompoundBorder(
	        BorderFactory.createEmptyBorder(0, 30, 0, 30), tb));
    
    DTDTextArea = new JTextArea(
		"To browse DTDs, first set the DTD directory\n"+
		"through the menu.  Then select a DTD from the\n"+
		"DTD list, and the DTD contents will be shown\n"+
		"here.\n");
    DTDTextArea.setEditable(false);
    DTDTextArea.setLineWrap(false);
    DTDTextArea.setWrapStyleWord(true);
    
    DTDScrollPane = new JScrollPane(DTDTextArea);
    tb = BorderFactory.createTitledBorder(
		   BorderFactory.createEtchedBorder(), "DTD View");
    tb.setTitleJustification(TitledBorder.CENTER);
    DTDScrollPane.setBorder(BorderFactory.createCompoundBorder(
		   BorderFactory.createEmptyBorder(5, 5, 5, 5), tb));
    
    splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
			       listScrollPane, DTDScrollPane);
    splitPane.setOneTouchExpandable(true);
    splitPane.setDividerLocation(100);
    contentPane.add(splitPane, BorderLayout.WEST);
    
    //adding Query Tag list ScrollPane
    list = new JList(queryTagModel);
    listScrollPane = new JScrollPane(list);
    tb = BorderFactory.createTitledBorder(
		  BorderFactory.createEtchedBorder(), "Query Tags");
    tb.setTitleJustification(TitledBorder.CENTER);
    listScrollPane.setBorder(BorderFactory.createCompoundBorder(
		BorderFactory.createEmptyBorder(2,2,2,2), tb));

    //adding Query ScrollPane
    query = new JTextArea("<?xml version=\"1.0\"?>\n", 8, 30);
    query.setEditable(true);
    query.setLineWrap(true);
    query.setWrapStyleWord(true);
    queryPane = new JScrollPane(query);
    tb = BorderFactory.createTitledBorder(
	        BorderFactory.createEtchedBorder(), "Current Query");
    tb.setTitleJustification(TitledBorder.CENTER);
    queryPane.setBorder(BorderFactory.createCompoundBorder(
		BorderFactory.createEmptyBorder(5, 5, 5, 5), tb));
    //adding splitPane between queryTags and queryView
    JSplitPane splitPane2 = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
					   listScrollPane, queryPane);
    splitPane2.setOneTouchExpandable(true);
    splitPane2.setDividerLocation(100);
    
    output = new JTextArea(20, 35);
    output.setEditable(false);
    scrollPane = new JScrollPane(output);
    tb = BorderFactory.createTitledBorder(
		 BorderFactory.createEtchedBorder(), "Output / Log");
    tb.setTitleJustification(TitledBorder.CENTER);
    scrollPane.setBorder(BorderFactory.createCompoundBorder(
		    BorderFactory.createEmptyBorder(5, 5, 5, 5), tb));
    //adding splitPane between query stuff and output/log view
    splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
			       splitPane2, scrollPane);
    splitPane.setOneTouchExpandable(true);
    splitPane.setDividerLocation(150);
    contentPane.add(splitPane, BorderLayout.CENTER);
    
    //Build Up MenuBar
    JMenuBar menuBar = new JMenuBar();
    menuBar.setOpaque(true);
    menuBar.setPreferredSize(new Dimension (200,20));
    
    JMenuItem menuItem;
    JMenu menu, submenu;
    
    //Build File Menu
    menu = new JMenu("File");
    menuBar.add(menu);
    menuItem = new JMenuItem("Add File");
    menuItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	String result;
	result = JOptionPane.showInputDialog(SwingDemoApp.this, 
			 "Please enter the filename including full path:");
	if (result != null && result.length() > 0) {
	  Object[] message = new Object[2];
	  message[0] = "Add file request sent.";
	  try {
	    xset.AddFile(result);
	    message[1] = "File successfully added.";
	    JOptionPane.showMessageDialog(SwingDemoApp.this, message);
	    output.append("File "+result+" added to dataset.\n");
	  } catch (java.rmi.RemoteException rRe) {
	    message[1] = "Add Error: "+rRe;
	    JOptionPane.showMessageDialog(SwingDemoApp.this, message);
	  }
	} else JOptionPane.showMessageDialog(SwingDemoApp.this, 
					     "No file selected for add.");
      }
    });  
    menu.add(menuItem);
    menuItem = new JMenuItem("Add XML String");
    menuItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	String result;
	result = JOptionPane.showInputDialog(SwingDemoApp.this, 
	     "Please enter XML String to be added to dataset:");
	if (result != null) {
	  Object[] message = new Object[2];
	  message[0] = "Add file request sent.";
	  try {
	    xset.AddDocument(result);
	    message[1] = "XML String successfully added.";
	    JOptionPane.showMessageDialog(SwingDemoApp.this, message);
	    output.append("XML String "+result+" added to dataset.\n");
	  } catch (java.rmi.RemoteException rRe) {
	    message[1] = "Add Error: "+rRe;
	    JOptionPane.showMessageDialog(SwingDemoApp.this, message);
	  }
	} else JOptionPane.showMessageDialog(SwingDemoApp.this, 
					     "No XML string entered for add.");
      }
    });  
    menu.add(menuItem);
    menuItem = new JMenuItem("Add Directory");
    menuItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	String result;
	result = JOptionPane.showInputDialog(SwingDemoApp.this, 
	   "Please enter full path of the directory to be added to dataset:");
	if (result != null) {
	  Object[] message = new Object[2];
	  message[0] = "Add directory request sent.";
	  try {
	    xset.AddFiles(result);
	    message[1] = "XML files successfully added.";
	    JOptionPane.showMessageDialog(SwingDemoApp.this, message);
	    output.append("Directory "+result+" added to dataset.\n");
	  } catch (java.rmi.RemoteException rRe) {
	    message[1] = "Add Error: "+rRe;
	    JOptionPane.showMessageDialog(SwingDemoApp.this, message);
	  }
	} else JOptionPane.showMessageDialog(SwingDemoApp.this, 
					     "No XML directory name entered.");
      }
    });
    menu.add(menuItem);
    menuItem = new JMenuItem("Clear XSet");
    menuItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	int result;
	result = JOptionPane.showConfirmDialog(SwingDemoApp.this, 
					       "Clear XSet Dataset?");
	if (result == JOptionPane.YES_OPTION) {
	  try {
	    xset.ClearAll();
	    JOptionPane.showMessageDialog(SwingDemoApp.this, 
					  "XSet cleared");
	    output.append("XSet dataset cleared.");
	  } catch (java.rmi.RemoteException rRe) {
	    JOptionPane.showMessageDialog(SwingDemoApp.this, 
		  "Exception during XSet clear operation.");
	  }
	} else JOptionPane.showMessageDialog(SwingDemoApp.this, 
					     "XSet clear operation canceled.");
      }
    });
    menu.add(menuItem);
    menuItem = new JMenuItem("Close");
    menuItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	System.exit(0);
      }
    });
    menuItem.setAccelerator(KeyStroke.getKeyStroke(
		   KeyEvent.VK_F4, ActionEvent.CTRL_MASK));
    menu.add(menuItem);
    
    //Build View Menu
    menu = new JMenu("DTDs");
    menuBar.add(menu);
    /*	submenu = new JMenu("View DTDs");
	menuItem = new JMenuItem("Printers");
	menuItem.addActionListener(this);
	submenu.add(menuItem);
	menuItem = new JMenuItem("Email AddressBook");
	menuItem.addActionListener(this);
	submenu.add(menuItem);
	menuItem = new JMenuItem("Department Roster");
	menuItem.addActionListener(this);
	submenu.add(menuItem);
	menu.add(submenu);
    */
    
    menuItem = new JMenuItem("Set DTD Dir");
    menuItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	String result;
	result = JOptionPane.showInputDialog(SwingDemoApp.this, 
		    "Please enter full path of the DTD storage directory:");
	if (result != null) {
	  if (getDTDs(result)) {
	    JOptionPane.showMessageDialog(SwingDemoApp.this, 
					  "DTD Directory Set.");
	    output.append("DTD Directory set as "+result+".\n");
	  } else 
	    JOptionPane.showMessageDialog(SwingDemoApp.this, 
					  "DTD Directory does not exist.");
	} else JOptionPane.showMessageDialog(SwingDemoApp.this, 
					     "No DTD directory entered.");
      }
    });
    menu.add(menuItem);
    
    //Build Query Menu
    menu = new JMenu("Query");
    menuBar.add(menu);
    menuItem = new JMenuItem("Perform Query");
    menuItem.setAccelerator(KeyStroke.getKeyStroke(
		   KeyEvent.VK_Q, ActionEvent.CTRL_MASK));
    menuItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	String queryTxt = query.getText();
	if (queryTxt == null || queryTxt.length() == 0) {
	  JOptionPane.showMessageDialog(SwingDemoApp.this,
					"No Query Specified.");
	  return;
	}
	//	    output.append(queryTxt+"\n");
	Vector queryResult;
	String result;
	if (queryTagModel.size() == 0) {
	  try {
	    queryResult = xset.Query(queryTxt);
	    if (queryResult.size() != 0) {
	      output.append("Found "+queryResult.size()+
			    " XML documents:\n");
	      Enumeration enu = queryResult.elements();
	      while (enu.hasMoreElements()) {
		result = (String) enu.nextElement();
		output.append(result+"\n");
	      }
	    } else output.append("No matching XML documents found.\n");
	  } catch (java.rmi.RemoteException jre) {
	    output.append("Error, Remote Exception thrown: "+
			  jre.getMessage()+"\n");
	  }
	} else {
	  Vector queryTags = new Vector();
	  Enumeration enu = queryTagModel.elements();
	  while (enu.hasMoreElements())
	    queryTags.addElement(enu.nextElement());
	  try {
	    queryResult = xset.Query(queryTxt, queryTags);
	    if (queryResult.size() == 0)
	      output.append("No matching XML documents found.\n");
	    else {
	      output.append("Found "+queryResult.size()+
			    " XML documents:\n");
	      Enumeration a, b, c;
	      a = queryResult.elements();
	      Hashtable h;
	      Vector v;
	      String key;
	      while (a.hasMoreElements()) {
		output.append("Matched Document:\n");
		h = (Hashtable) a.nextElement();
		b = h.keys();
		while (b.hasMoreElements()) {
		  key = (String) b.nextElement();
		  output.append("  Tag "+key+": "+
				((Vector) h.get(key)).toString()+
				"\n");
		}
	      }
	    }
	  } catch (java.rmi.RemoteException jre) {
	    output.append("Error, Remote Exception thrown: "+
			  jre.getMessage()+"\n");
	  }
	  //convert queryTagModel to Vector,
	  //do query with return tags
	}
	// output results
      }
    });
    menu.add(menuItem);
    menuItem = new JMenuItem("Add Query Tag");
    menuItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	String result;
	result = JOptionPane.showInputDialog(SwingDemoApp.this, 
					     "Enter desired tag: ");
	if (result != null && result.length() > 0) {
	  queryTagModel.addElement(result);
	  JOptionPane.showMessageDialog(SwingDemoApp.this, 
					"Tag "+result+" added.");
	  output.append("Tag "+result+" added to query tag list.\n");
	} else JOptionPane.showMessageDialog(SwingDemoApp.this, 
					     "No tag added.");
      }
    });  
    menu.add(menuItem);
    menuItem = new JMenuItem("Remove Query Tag");
    menuItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	String result;
	result = JOptionPane.showInputDialog(SwingDemoApp.this, 
					     "Enter tag to be removed: ");
	if (result != null && result.length() > 0) {
	  if (queryTagModel.removeElement(result)) {
	    JOptionPane.showMessageDialog(SwingDemoApp.this, 
					  "Tag "+result+" removed.");
	    output.append("Tag "+result+" removed from query tags.\n");
	  } else JOptionPane.showMessageDialog(SwingDemoApp.this,
					       "Tag "+result+" not found.");
	} else JOptionPane.showMessageDialog(SwingDemoApp.this, 
					     "No tag removed.");
      }
    });  
    menu.add(menuItem);
    menuItem = new JMenuItem("Clear Query Tags");
    menuItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	queryTagModel.clear();
	JOptionPane.showMessageDialog(SwingDemoApp.this, 
				      "All query tags cleared.");
	output.append("All query tags cleared.\n");
      }
    });  
    menu.add(menuItem);
    
    //Build Help Menu
    menu = new JMenu("Help");
    menuBar.add(menu);
    menuItem = new JMenuItem("About XSet Swing Demo");
    menuItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	Object[] message = new Object[4];
	message[0] = "Swing Demo for the XSet Search Engine";
	message[1] = "Version 1.1, June, 1999";
	message[2] = "(C) 1999, Ben Yanbin Zhao and the Regents"+
	  " of University of California";
	message[3] = "Freely distributable under the Berkeley license";
	JOptionPane.showMessageDialog(SwingDemoApp.this, message);
      }
    });
    menu.add(menuItem);
    menuItem = new JMenuItem("About XSet");
    menuItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	Object[] message = new Object[4];
	message[0] = "The XSet XML Search Engine: V. 1.1, June 1999";
	message[1] = "URL: http://www.cs.berkeley.edu/~ravenben/xset";
	message[2] = "(C) 1999, Ben Yanbin Zhao and the Regents"+
	  " of University of California";
	message[3] = "Freely distributable under the Berkeley license";
	JOptionPane.showMessageDialog(SwingDemoApp.this, message);
      }
    });
    menu.add(menuItem);
    
    //Adding components to Frame
    setJMenuBar(menuBar);
  }
  
  protected boolean getDTDs(String dir) {
    File dtddir = new File(dir);
    String [] dtdlist;
    if ((dtdlist = dtddir.list(new ExtensionFilter("dtd"))) == null)
      return false;
    DTDPath = dir;
    String dtdString;
    for (int i = 0; i < dtdlist.length; i++) {
      DTDNames.addElement(dtdlist[i]);
      try {
	BufferedReader br = new BufferedReader(new FileReader(dir+"/"+dtdlist[i]));
	dtdString = "";
	String tmp = null;
	while ((tmp = br.readLine()) != null) {
	  dtdString += tmp+"\n";
	}
      } catch (Exception e) {
	e.printStackTrace(System.err);
	continue;
      }
      DTDContents.put(dtdlist[i], dtdString);
      output.append("DTD added: "+dtdlist[i]+"\n");
    }
    return true;
  }
  
  public void actionPerformed(ActionEvent e) {
    JMenuItem source = (JMenuItem) e.getSource();
    String src = source.getText();
    //	if (src.equals("Close")) {
    //	    System.exit(0);
    //	} 
    output.append("Item Event:\nSource: "+source.getText()+"\n");
  }
  
  //    public void itemStateChanged(ItemEvent e) {
  //FILL IN
  //    }
  
  public void valueChanged(ListSelectionEvent e) {
    ListSelectionModel lsm = (ListSelectionModel) 
      ((JList) e.getSource()).getSelectionModel();
    int firstIndex = e.getFirstIndex();
    int lastIndex = e.getLastIndex();
    int changedIndex = -1;
    for (int i = firstIndex; i <= lastIndex; i++) {
      if (lsm.isSelectedIndex(i)) {
	changedIndex = i;
	break;
      }
    }
    if (changedIndex == -1) return;
    String newDTD = (String) DTDNames.elementAt(changedIndex);
    String newDTDView = (String) DTDContents.get(newDTD);
    DTDTextArea.setText(newDTDView);
  }
  
  public static void main(String argv[]) {
    if (argv.length < 1) {
      usage();
      System.exit(0);
    }
    SwingDemoApp window = new SwingDemoApp(argv[0]);
    window.setTitle("XSet Swing Demo");
    window.setSize(900, 550);
    window.setVisible(true);
  }
  public static void usage() {
    System.err.println("Usage: java ninja.xset.SwingDemoApp hostname");
  }
}
