/**
 * @author: Ben Y. Zhao (ravenben@cs.berkeley.edu)
 ** @version: Release 1.1 CVS:$Revision: 1.12 $ $Date: 1999/09/10 00:12:35 $
 ** (c) 1999 by Ben Yanbin Zhao and the Regents of the University of California
 **/

package ninja.xset;

import java.io.*;
import java.util.*;
import ninja.ispace.*;
//import ninja.xset.XSetServiceIF;

/**
 ** A sample client class for the XSet project
 **/
public class XSetClient {
  iSpaceLoaderIF loader;
  XSetServiceIF XSetService;
  String hostname = null;
  String path = null;
  String query = "<?xml version=\"1.0\"?><PRINTCAP><COLOR>YES</COLOR></PRINTCAP>";

  /**
   ** simple constructor that initializes two fields according to 
   ** cmdline arguments
   **/
  public XSetClient(String host, String xmlpath) {
    hostname = host;
    path = xmlpath;
  }

  /**
   ** Main code is here
   ** The code simply calls XSet to index the files in the XML path,
   ** and then does a query on them with a hard-coded XML query fragment.
   **/
  public void run() {
    String [] servlist;
    Vector results;
    try {
      loader = (iSpaceLoaderIF)iSpaceLoader.getLoaderStub(hostname);
    } catch (Exception e) {
      System.err.println("Can't get loader: "+e);
      //      return;
    }
    try {
      servlist = loader.listServices();
      for (int i = 0; i < servlist.length; i++) {
	System.out.println(servlist[i]);
      }
      XSetService = (XSetServiceIF) loader.getService("ninja.xset.XSetService");
      if (XSetService == null) System.out.println("UTOH");
      XSetService.AddFiles(path);
      results = XSetService.Query(query);
      System.out.println("Result: "+results.toString());
      Vector v = new Vector();
      v.addElement("NAME");
      results = XSetService.Query(query, v);
      if (results.size() == 0) System.out.println("No Matches.");
      else {
	System.out.println("Matches:");
	Hashtable ht = (Hashtable) results.elementAt(0);
	String out = (String) ((Vector) ht.get("NAME")).toString();
	System.out.println("Match: "+out);
      }
      String [] tagPath = new String [2];
      tagPath[0] = "PRINTCAP";
      tagPath[1] = "ROOM";
      v = XSetService.EnumerateTag(tagPath);
      System.out.println("Enumeration for room: "+v.toString());
      XSetService.ClearAll();
      results = XSetService.Query(query);
      if (results == null) System.out.println("No Matches.");
      else System.out.println("Matches: "+results.toString());
    } catch (Exception e) {
      System.out.println("Exception while execution: "+e);
      e.printStackTrace();
    }
  }
  
  public static void main(String argv[]) {
    if (argv.length != 2) {
      System.out.println("Wrong # of arguments!");
      System.out.println("  java ninja.xset.XSetClient hostname pathtoxmlfiles");
      System.out.println("e.g.: java ninja.xset.XSetClient joker.cs.berkeley.edu /usr/local/xmlrepository/");
      return;
    }
    XSetClient xs = new XSetClient(argv[0], argv[1]);
    xs.run();
  }
}
