/**
 * @author: Ben Y. Zhao (ravenben@cs.berkeley.edu)
 ** @version: Release 1.1 CVS:$Revision: 1.4 $ $Date: 1999/09/10 00:12:35 $
 * (c) 1999 by Ben Yanbin Zhao and the Regents of the University of California
 */

package ninja.xset;

import java.io.*;
import java.util.Vector;
import java.util.Enumeration;
import java.rmi.RemoteException;
import org.w3c.dom.*;
import com.ibm.xml.parsers.DOMParser;
import org.xml.sax.InputSource;

public class XSetDBBushy {
  private SETserver server;
  private Runtime run;

  private int filesindexed;
  private long indextime;
  private long querytime;
  private int memoryleft;
  private int memorytotal;
  String query1 = "<?xml version=\"1.0\"?><BOOKENTRY><DISCOUNT>30%</DISCOUNT></BOOKENTRY>";
  String query2 = "<?xml version=\"1.0\"?><BOOKENTRY><PUBLISHED><YEAR>1998</YEAR></PUBLISHED><DISCOUNT>30%</DISCOUNT></BOOKENTRY>";
  String query3 = "<?xml version=\"1.0\"?><BOOKENTRY><PUBLISHED><BY>McGRAW-HILL</BY><YEAR>1998</YEAR></PUBLISHED><DISCOUNT>30%</DISCOUNT></BOOKENTRY>";

  public XSetDBBushy() {
  }

  public static void main(String [] argv) {
    XSetDBBushy xs = new XSetDBBushy();
    xs.run();
  }
  
  public void run() {
    System.out.println("XSetService: started.");
    int load = 20;
    filesindexed = 0;
    run = Runtime.getRuntime();
    try {
      XSetClearAll();
      XSetAddFiles("/usr/home/research/xset/books");
     } catch (Exception ex) {ex.printStackTrace();}
    Vector ans = null;
    querytime = System.currentTimeMillis();
    for (int j = 0; j<10; j++) {
	ans = XSetQuery(query1);
    }
    querytime = System.currentTimeMillis() - querytime;
    System.out.println("10 runs of query1: "+querytime+" "+ans.size());

    querytime = System.currentTimeMillis();
    for (int j = 0; j<10; j++) {
	ans = XSetQuery(query2);
    }
    querytime = System.currentTimeMillis() - querytime;
    System.out.println("10 runs of query1: "+querytime+" "+ans.size());

    querytime = System.currentTimeMillis();
    for (int j = 0; j<10; j++) {
	ans = XSetQuery(query3);
    }
    querytime = System.currentTimeMillis() - querytime;
    System.out.println("10 runs of query1: "+querytime+" "+ans.size());
  }

  public void XSetAddFiles(String path) {
    //    System.out.println("XSetAddFiles request received!");
    File dir = new File(path);
    String [] xmlFiles = null;
    Document d = null;

    if ((xmlFiles = dir.list(new ExtensionFilter("xml"))) == null) {
      System.out.println("Directory not found or inaccessible!");
      return;
    }
    int i = 0;
    try {
	DOMParser parser = new DOMParser();
	for (i = 0; i < xmlFiles.length; i++) {
	    //	System.out.println("File: "+xmlFiles[i]+ " loaded.");
	    parser.parse(path+"/"+xmlFiles[i]);
	    d = parser.getDocument();
	    server.SETregister(d);
	    parser.reset();
	}
    } catch (Exception e) {
	e.printStackTrace();
	System.err.println("Query file not found!");
	return;
    }
  }
    
  public void XSetClearAll() {
    System.out.println("XSetClearAll request received!");
    server = new SETserver();
    System.out.println("XSet cleared.");
  }

  public Vector XSetQuery(String xmlQuery) {
    //    System.out.println("XSetQuery request received!");
    Document d;
    Vector results = null;
    try {
      DOMParser parser = new DOMParser();
      ByteArrayInputStream bais=new ByteArrayInputStream(xmlQuery.getBytes());
      parser.parse(new InputSource(bais));
      d = parser.getDocument();
      results = server.querySET(d);
    } catch (Exception e) {
      e.printStackTrace();
      System.err.println("Error parsing query string: \n"+xmlQuery);
    }
    return results;
  }
}
