/**
 * @author: Ben Y. Zhao (ravenben@cs.berkeley.edu)
 ** @version: Release 1.1 CVS:$Revision: 1.4 $ $Date: 1999/09/10 00:12:36 $
 * (c) 1999 by Ben Yanbin Zhao and the Regents of the University of California
 */

package ninja.xset;

import java.io.*;
import java.util.Vector;
import java.util.Enumeration;
import java.rmi.RemoteException;
import com.ms.xml.om.*;
import com.ms.xml.util.*;

public class XSetDBIndex {
  private SETserver server;
  private Runtime run;

  private int filesindexed;
  private long indextime;
  private long totaltime;
  private long querytime;
  private int memoryleft;
  private int memorytotal;
  String query1 = "<?xml version=\"1.0\"?><WEBLOG><SOURCEIP>lvl-sun669.usc.edu</SOURCEIP></WEBLOG>";
  String query2 = "<?xml version=\"1.0\"?><WEBLOG><SOURCEIP>crawl3.atext.com</SOURCEIP></WEBLOG>";
  String query3 = "<?xml version=\"1.0\"?><WEBLOG><SOURCEIP>intergate.sbhsd.k12.ca.us</SOURCEIP></WEBLOG>";
  String query4 = "<?xml version=\"1.0\"?><WEBLOG><SOURCEIP>p18-term1-and.netdirect.net</SOURCEIP></WEBLOG>";
  String query5 = "<?xml version=\"1.0\"?><WEBLOG><SOURCEIP>209.101.58.234</SOURCEIP></WEBLOG>";
  String query6 = "<?xml version=\"1.0\"?><WEBLOG><SOURCEIP>novartis2-bh.us.novartis.com</SOURCEIP></WEBLOG>";
  String query7 = "<?xml version=\"1.0\"?><WEBLOG><SOURCEIP>kerkyra.epsilon.com.gr</SOURCEIP></WEBLOG>";
  String query8 = "<?xml version=\"1.0\"?><WEBLOG><SOURCEIP>lsi-station1.lsi.dsc.ufpb.br</SOURCEIP></WEBLOG>";
  String query9 = "<?xml version=\"1.0\"?><WEBLOG><SOURCEIP>ckc-114-60.reshall.berkeley.edu</SOURCEIP></WEBLOG>";
  String query10 = "<?xml version=\"1.0\"?><WEBLOG><SOURCEIP>205.238.147.244</SOURCEIP></WEBLOG>";

  public XSetDBIndex() {
  }

  public static void main(String [] argv) {
    XSetDBIndex xs = new XSetDBIndex();
    xs.run();
  }
  
  public void run() {
    System.out.println("XSetService: started.");
    int load = 20;
    totaltime = 0;
    filesindexed = 0;
    run = Runtime.getRuntime();
    try {
      XSetClearAll();
      for (int i =0; i<load; i++) {
	  XSetAddFiles("/usr/home/research/xset/db/"+i);
      }
      /*      long time;
      Vector ans = null;
      System.out.println("Database loaded with "+load+" directories\n");
      time = System.currentTimeMillis();
      for (int i =0; i<100; i++) {
        ans = XSetQuery(query);
      }
      time = System.currentTimeMillis() - time;
      System.out.println("1 Xset query: "+time);
      time = System.currentTimeMillis();
      for (int i =0; i<100; i++) {
	ans = XSetQuery(query);
      }
      time = System.currentTimeMillis() - time;
      System.out.println("2nd Xset query: "+time);
      if (ans == null) return;
      Enumeration e = ans.elements();
      while (e.hasMoreElements()) {
	  System.out.println("Result: >>"+
			     ((ServiceEntry)e.nextElement()).getXML());
      }
      */
     } catch (Exception ex) {ex.printStackTrace();}
  }

  public void XSetAddFiles(String path) {
    //    System.out.println("XSetAddFiles request received!");
    File dir = new File(path);
    String [] xmlFiles = null;
    Document d = null;

    if ((xmlFiles = dir.list(new ExtensionFilter("xml"))) == null) {
      System.out.println("Directory not found or inaccessible!");
      return;
    }
    int i = 0;
    try {
      for (i = 0; i < xmlFiles.length; i++) {
	//	System.out.println("File: "+xmlFiles[i]+ " loaded.");
	d = new Document();
	d.load(SETutils.parseFile(path+"/"+xmlFiles[i]));
	indextime = System.currentTimeMillis();
	server.SETregister(d);
	totaltime += System.currentTimeMillis() - indextime;
	if ((i+1) % 100 == 0) {
	    System.out.println("Documents, 100-index: "+(filesindexed+1)+"##"+
			       totaltime);
	    totaltime = 0;
	}
	filesindexed++;
      }
    } catch (FileNotFoundException fnfe) {
      System.err.println("Query file not found!");
      return;
    } catch (com.ms.xml.parser.ParseException pe) {
      System.err.println("Parsing ERROR on: "+xmlFiles[i]);
    }
  }

  public void XSetClearAll() {
    System.out.println("XSetClearAll request received!");
    server = new SETserver();
    System.out.println("XSet cleared.");
  }

  public Vector XSetQuery(String xmlQuery) {
    //    System.out.println("XSetQuery request received!");
    Document d = new Document();
    Vector results = null;
    InputStream is = new ByteArrayInputStream(xmlQuery.getBytes());
    try {
      d.load(is);
      results = server.querySET(d);
    } catch (com.ms.xml.parser.ParseException pe) {
      System.err.println("Error parsing query string: \n"+xmlQuery);
    }
    return results;
  }
}
