/**
 ** @author: Ben Y. Zhao (ravenben@cs.berkeley.edu)
 ** @version: Release 1.1 CVS:$Revision: 1.9 $ $Date: 1999/09/10 00:12:36 $
 ** (c) 1999 by Ben Yanbin Zhao and the Regents of the University of California
 ** <P>
 ** This is a simple file that runs a couple of printer queries using SETserver
 **/

package ninja.xset;

import java.io.*;
import java.util.Vector;
import java.util.Enumeration;
import java.rmi.RemoteException;
import org.w3c.dom.*;
import com.ibm.xml.parsers.DOMParser;

public class XSetProfile {
  private SETserver server;
  Runtime run;
  
  public XSetProfile() {
  }

  public static void main(String [] argv) {
    XSetProfile xs = new XSetProfile();
    xs.run();
  }
  
  public void run() {
    run = Runtime.getRuntime();
    System.out.println("XSetService: started.");
    try {
      XSetClearAll();
      XSetAddFiles("/home/ravenben/ninja/classpath/ninja/xset/printcaps");
      String query = "<?xml version=\"1.0\"?><PRINTCAP><COLOR>NO</COLOR></PRINTCAP>";

      String [] tagPath = new String [3];
      tagPath[0] = "PRINTCAP";
      tagPath[1] = "REMOTE";
      tagPath[2] = "SERVER";
      Enumeration enu = server.enumerateTag(tagPath);
      String out = "";
      while (enu.hasMoreElements()) {
	out = out +" "+enu.nextElement();
      }
      System.out.println("Enumeration for room: "+out);
      
      long time;
      Vector ans = null;
      time = System.currentTimeMillis();
      for (int i =0; i<100; i++) {
	ans = XSetQuery(query);
      }
      time = System.currentTimeMillis() - time;
      System.out.println("1 Xset query with 1 constraint: "+time);
      if (ans == null) return;
      Enumeration e = ans.elements();
      while (e.hasMoreElements()) {
	System.out.println("Result: >>"+
			   ((ServiceEntry)e.nextElement()).getXML());
      }
      query = "<?xml version=\"1.0\"?><PRINTCAP><DUPLEX>YES</DUPLEX><COLOR>NO</COLOR></PRINTCAP>";
      time = System.currentTimeMillis();
      for (int i =0; i<100; i++) {
	ans = XSetQuery(query);
      }
      time = System.currentTimeMillis() - time;
      System.out.println("1 Xset query with 2 constraints: "+time);
      if (ans == null) return;
      e = ans.elements();
      while (e.hasMoreElements()) {
	System.out.println("Result: >>"+
			   ((ServiceEntry)e.nextElement()).getXML());
      }
    } catch (Exception ex) {ex.printStackTrace();}
  }
  
  public void XSetAddFiles(String path) {
    //    System.out.println("XSetAddFiles request received!");
    File dir = new File(path);
    String [] xmlFiles = null;
    Document d = null;
    
    if ((xmlFiles = dir.list(new ExtensionFilter("xml"))) == null) {
      System.out.println("Directory not found or inaccessible!");
      return;
    }
    int i = 0;
    try {
      DOMParser parser = new DOMParser();
      //      for (i = 0; i < 1; i++) {
      for (i = 0; i < xmlFiles.length; i++) {
	System.out.println("File: "+xmlFiles[i]+ " loading.");
	parser.parse(path+"/"+xmlFiles[i]);
	d = parser.getDocument();
	server.SETregister(d);
	parser.reset();
	System.out.println(xmlFiles[i]+" indexed.\n");
	System.out.println("FreeMemory: "+run.freeMemory()+ "totalMemory: "+
			   run.totalMemory()+"\n");
	run.gc();
	System.out.println("FreeMemory: "+run.freeMemory()+ "totalMemory: "+
			   run.totalMemory()+"\n");
      }
    } catch (FileNotFoundException fnfe) {
      System.err.println("Query file not found!");
      return;
    } catch (org.xml.sax.SAXException saxe) {
      saxe.printStackTrace(System.err);
    } catch (java.io.IOException ioe) {
      ioe.printStackTrace(System.err);
    } catch (XSetException xse) {
      xse.printStackTrace(System.err);
    }
  }

  public void XSetClearAll() {
    System.out.println("XSetClearAll request received!");
    server = new SETserver();
    System.out.println("XSet cleared.");
  }

  public Vector XSetQuery(String xmlQuery) {
    //    System.out.println("XSetQuery request received!");
    Document d;
    DOMParser parser = new DOMParser();
    Vector results = null;
    InputStream is = new ByteArrayInputStream(xmlQuery.getBytes());
    try {
      parser.parse(new org.xml.sax.InputSource(is));
      d = parser.getDocument();
      results = server.querySET(d);
    } catch (org.xml.sax.SAXException saxe) {
      saxe.printStackTrace(System.err);
    } catch (java.io.IOException ioe) {
      ioe.printStackTrace(System.err);
    }
    return results;
  }
}
